@@ -323,6 +323,33 @@ ApplyTransformationMatrix(DeviceIntPtr dev)
                            PropModeReplace, 9, matrix, FALSE);
 }
 
+static void
+ApplyAutoRepeat(DeviceIntPtr dev)
+{
+    InputInfoPtr pInfo = (InputInfoPtr) dev->public.devicePrivate;
+    XkbSrvInfoPtr xkbi;
+    char *repeatStr;
+    long delay, rate;
+
+    if (!dev->key)
+        return;
+
+    xkbi = dev->key->xkbInfo;
+
+    repeatStr = xf86SetStrOption(pInfo->options, "AutoRepeat", NULL);
+    if (!repeatStr)
+        return;
+
+    if (sscanf(repeatStr, "%ld %ld", &delay, &rate) != 2) {
+        xf86Msg(X_ERROR, "\"%s\" is not a valid AutoRepeat value\n", repeatStr);
+        return;
+    }
+
+    xf86Msg(X_CONFIG, "AutoRepeat: %ld %ld\n", delay, rate);
+    xkbi->desc->ctrls->repeat_delay = delay;
+    xkbi->desc->ctrls->repeat_interval = rate;
+}
+
 /***********************************************************************
  *
  * xf86ProcessCommonOptions --
@@ -821,6 +848,7 @@ xf86InputDevicePostInit(DeviceIntPtr dev)
 {
     ApplyAccelerationSettings(dev);
     ApplyTransformationMatrix(dev);
+    ApplyAutoRepeat(dev);
     return Success;
 }
 
@@ -874,7 +902,7 @@ xf86NewInputDevice(InputInfoPtr pInfo, DeviceIntPtr *pdev, BOOL enable)
 {
     InputDriverPtr drv = NULL;
     DeviceIntPtr dev = NULL;
-    Bool paused;
+    Bool paused = FALSE;
     int rval;
     char *path = NULL;
 
@@ -1181,7 +1209,7 @@ xf86CheckMotionEvent4DGA(DeviceIntPtr device, int is_absolute,
 {
     int stolen = 0;
 
-#if XFreeXDGA
+#ifdef XFreeXDGA
     ScreenPtr scr = NULL;
     int idx = 0, i;
 
@@ -1375,7 +1403,7 @@ xf86PostButtonEventM(DeviceIntPtr device,
             flags = POINTER_RELATIVE | POINTER_ACCELERATE;
     }
 
-#if XFreeXDGA
+#ifdef XFreeXDGA
     if (miPointerGetScreen(device)) {
         int index = miPointerGetScreen(device)->myNum;
 
@@ -1406,7 +1434,7 @@ xf86PostKeyEventP(DeviceIntPtr device,
 void
 xf86PostKeyEventM(DeviceIntPtr device, unsigned int key_code, int is_down)
 {
-#if XFreeXDGA
+#ifdef XFreeXDGA
     DeviceIntPtr pointer;
 
     /* Some pointers send key events, paired device is wrong then. */
@@ -1448,7 +1476,7 @@ xf86FirstLocalDevice(void)
  *
  * This function is the same for X or Y coordinates.
  * You may have to reverse the high and low values to compensate for
- * different orgins on the touch screen vs X.
+ * different origins on the touch screen vs X.
  *
  * e.g. to scale from device coordinates into screen coordinates, call
  * xf86ScaleAxis(x, 0, screen_width, dev_min, dev_max);
@@ -1562,6 +1590,58 @@ xf86PostTouchEvent(DeviceIntPtr dev, uint32_t touchid, uint16_t type,
     QueueTouchEvents(dev, type, touchid, flags, mask);
 }
 
+/**
+ * Post a gesture pinch event.  The driver is responsible for maintaining the
+ * correct event sequence (GesturePinchBegin, GesturePinchUpdate,
+ * GesturePinchEnd).
+ *
+ * @param dev The device to post the event for
+ * @param type One of XI_GesturePinchBegin, XI_GesturePinchUpdate,
+ *        XI_GesturePinchEnd
+ * @param num_touches The number of touches in the gesture
+ * @param flags Flags for this event
+ * @param delta_x,delta_y accelerated relative motion delta
+ * @param delta_unaccel_x,delta_unaccel_y unaccelerated relative motion delta
+ * @param scale absolute scale of a pinch gesture
+ * @param delta_angle the ange delta in degrees between the last and the current pinch event.
+ */
+void
+xf86PostGesturePinchEvent(DeviceIntPtr dev, uint16_t type,
+                          uint16_t num_touches, uint32_t flags,
+                          double delta_x, double delta_y,
+                          double delta_unaccel_x,
+                          double delta_unaccel_y,
+                          double scale, double delta_angle)
+{
+    QueueGesturePinchEvents(dev, type, num_touches, flags, delta_x, delta_y,
+                            delta_unaccel_x, delta_unaccel_y,
+                            scale, delta_angle);
+}
+
+/**
+ * Post a gesture swipe event.  The driver is responsible for maintaining the
+ * correct event sequence (GestureSwipeBegin, GestureSwipeUpdate,
+ * GestureSwipeEnd).
+ *
+ * @param dev The device to post the event for
+ * @param type One of XI_GestureSwipeBegin, XI_GestureSwipeUpdate,
+ *        XI_GestureSwipeEnd
+ * @param num_touches The number of touches in the gesture
+ * @param flags Flags for this event
+ * @param delta_x,delta_y accelerated relative motion delta
+ * @param delta_unaccel_x,delta_unaccel_y unaccelerated relative motion delta
+ */
+void
+xf86PostGestureSwipeEvent(DeviceIntPtr dev, uint16_t type,
+                          uint16_t num_touches, uint32_t flags,
+                          double delta_x, double delta_y,
+                          double delta_unaccel_x,
+                          double delta_unaccel_y)
+{
+    QueueGestureSwipeEvents(dev, type, num_touches, flags, delta_x, delta_y,
+                            delta_unaccel_x, delta_unaccel_y);
+}
+
 void
 xf86InputEnableVTProbe(void)
 {
