@@ -38,14 +38,12 @@
 #ifdef WITH_LIBDRM
 #include <xf86drm.h>
 #endif
-#include "xf86Module.h"
 #include "list.h"
 #include "scrnintstr.h"
 #include "windowstr.h"
 #include "dixstruct.h"
 #include "dri2.h"
 #include "dri2int.h"
-#include "xf86VGAarbiter.h"
 #include "damage.h"
 #include "xf86.h"
 
@@ -100,7 +98,7 @@ typedef struct _DRI2Drawable {
     unsigned blocked[3];
     Bool needInvalidate;
     int prime_id;
-    PixmapPtr prime_slave_pixmap;
+    PixmapPtr prime_secondary_pixmap;
     PixmapPtr redirectpixmap;
 } DRI2DrawableRec, *DRI2DrawablePtr;
 
@@ -183,33 +181,33 @@ DRI2GetScreen(ScreenPtr pScreen)
 }
 
 static ScreenPtr
-GetScreenPrime(ScreenPtr master, int prime_id)
+GetScreenPrime(ScreenPtr primary, int prime_id)
 {
-    ScreenPtr slave;
+    ScreenPtr secondary;
     if (prime_id == 0) {
-        return master;
+        return primary;
     }
-    xorg_list_for_each_entry(slave, &master->slave_list, slave_head) {
+    xorg_list_for_each_entry(secondary, &primary->secondary_list, secondary_head) {
         DRI2ScreenPtr ds;
 
-        if (!slave->is_offload_slave)
+        if (!secondary->is_offload_secondary)
             continue;
 
-        ds = DRI2GetScreen(slave);
+        ds = DRI2GetScreen(secondary);
         if (ds == NULL)
             continue;
 
         if (ds->prime_id == prime_id)
-            return slave;
+            return secondary;
     }
-    return master;
+    return primary;
 }
 
 static DRI2ScreenPtr
-DRI2GetScreenPrime(ScreenPtr master, int prime_id)
+DRI2GetScreenPrime(ScreenPtr primary, int prime_id)
 {
-    ScreenPtr slave = GetScreenPrime(master, prime_id);
-    return DRI2GetScreen(slave);
+    ScreenPtr secondary = GetScreenPrime(primary, prime_id);
+    return DRI2GetScreen(secondary);
 }
 
 static DRI2DrawablePtr
@@ -264,7 +262,7 @@ DRI2AllocateDrawable(DrawablePtr pDraw)
     xorg_list_init(&pPriv->reference_list);
     pPriv->needInvalidate = FALSE;
     pPriv->redirectpixmap = NULL;
-    pPriv->prime_slave_pixmap = NULL;
+    pPriv->prime_secondary_pixmap = NULL;
     if (pDraw->type == DRAWABLE_WINDOW) {
         pWin = (WindowPtr) pDraw;
         dixSetPrivate(&pWin->devPrivates, dri2WindowPrivateKey, pPriv);
@@ -358,10 +356,15 @@ DRI2CreateDrawable2(ClientPtr client, DrawablePtr pDraw, XID id,
                     XID *dri2_id_out)
 {
     DRI2DrawablePtr pPriv;
-    DRI2ClientPtr dri2_client = dri2ClientPrivate(client);
+    DRI2ClientPtr dri2_client;
     XID dri2_id;
     int rc;
 
+    if (!dixPrivateKeyRegistered(dri2ScreenPrivateKey))
+        return BadValue;
+
+    dri2_client = dri2ClientPrivate(client);
+
     pPriv = DRI2GetDrawable(pDraw);
     if (pPriv == NULL)
         pPriv = DRI2AllocateDrawable(pDraw);
@@ -429,9 +432,9 @@ DRI2DrawableGone(void *p, XID id)
         dixSetPrivate(&pPixmap->devPrivates, dri2PixmapPrivateKey, NULL);
     }
 
-    if (pPriv->prime_slave_pixmap) {
-        (*pPriv->prime_slave_pixmap->master_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_slave_pixmap->master_pixmap);
-        (*pPriv->prime_slave_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_slave_pixmap);
+    if (pPriv->prime_secondary_pixmap) {
+        (*pPriv->prime_secondary_pixmap->primary_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_secondary_pixmap->primary_pixmap);
+        (*pPriv->prime_secondary_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_secondary_pixmap);
     }
 
     if (pPriv->buffers != NULL) {
@@ -817,10 +820,10 @@ DrawablePtr DRI2UpdatePrime(DrawablePtr pDraw, DRI2BufferPtr pDest)
     DRI2DrawablePtr pPriv = DRI2GetDrawable(pDraw);
     PixmapPtr spix;
     PixmapPtr mpix = GetDrawablePixmap(pDraw);
-    ScreenPtr master, slave;
+    ScreenPtr primary, secondary;
     Bool ret;
 
-    master = mpix->drawable.pScreen;
+    primary = mpix->drawable.pScreen;
 
     if (pDraw->type == DRAWABLE_WINDOW) {
         WindowPtr pWin = (WindowPtr)pDraw;
@@ -833,15 +836,15 @@ DrawablePtr DRI2UpdatePrime(DrawablePtr pDraw, DRI2BufferPtr pDest)
                 pPriv->redirectpixmap->drawable.depth == pDraw->depth) {
                 mpix = pPriv->redirectpixmap;
             } else {
-                if (master->ReplaceScanoutPixmap) {
-                    mpix = (*master->CreatePixmap)(master, pDraw->width, pDraw->height,
+                if (primary->ReplaceScanoutPixmap) {
+                    mpix = (*primary->CreatePixmap)(primary, pDraw->width, pDraw->height,
                                                    pDraw->depth, CREATE_PIXMAP_USAGE_SHARED);
                     if (!mpix)
                         return NULL;
 
-                    ret = (*master->ReplaceScanoutPixmap)(pDraw, mpix, TRUE);
+                    ret = (*primary->ReplaceScanoutPixmap)(pDraw, mpix, TRUE);
                     if (ret == FALSE) {
-                        (*master->DestroyPixmap)(mpix);
+                        (*primary->DestroyPixmap)(mpix);
                         return NULL;
                     }
                     pPriv->redirectpixmap = mpix;
@@ -849,31 +852,31 @@ DrawablePtr DRI2UpdatePrime(DrawablePtr pDraw, DRI2BufferPtr pDest)
                     return NULL;
             }
         } else if (pPriv->redirectpixmap) {
-            (*master->ReplaceScanoutPixmap)(pDraw, pPriv->redirectpixmap, FALSE);
-            (*master->DestroyPixmap)(pPriv->redirectpixmap);
+            (*primary->ReplaceScanoutPixmap)(pDraw, pPriv->redirectpixmap, FALSE);
+            (*primary->DestroyPixmap)(pPriv->redirectpixmap);
             pPriv->redirectpixmap = NULL;
         }
     }
 
-    slave = GetScreenPrime(pDraw->pScreen, pPriv->prime_id);
+    secondary = GetScreenPrime(pDraw->pScreen, pPriv->prime_id);
 
     /* check if the pixmap is still fine */
-    if (pPriv->prime_slave_pixmap) {
-        if (pPriv->prime_slave_pixmap->master_pixmap == mpix)
-            return &pPriv->prime_slave_pixmap->drawable;
+    if (pPriv->prime_secondary_pixmap) {
+        if (pPriv->prime_secondary_pixmap->primary_pixmap == mpix)
+            return &pPriv->prime_secondary_pixmap->drawable;
         else {
-            PixmapUnshareSlavePixmap(pPriv->prime_slave_pixmap);
-            (*pPriv->prime_slave_pixmap->master_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_slave_pixmap->master_pixmap);
-            (*slave->DestroyPixmap)(pPriv->prime_slave_pixmap);
-            pPriv->prime_slave_pixmap = NULL;
+            PixmapUnshareSecondaryPixmap(pPriv->prime_secondary_pixmap);
+            (*pPriv->prime_secondary_pixmap->primary_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_secondary_pixmap->primary_pixmap);
+            (*secondary->DestroyPixmap)(pPriv->prime_secondary_pixmap);
+            pPriv->prime_secondary_pixmap = NULL;
         }
     }
 
-    spix = PixmapShareToSlave(mpix, slave);
+    spix = PixmapShareToSecondary(mpix, secondary);
     if (!spix)
         return NULL;
 
-    pPriv->prime_slave_pixmap = spix;
+    pPriv->prime_secondary_pixmap = spix;
 #ifdef COMPOSITE
     spix->screen_x = mpix->screen_x;
     spix->screen_y = mpix->screen_y;
@@ -1364,9 +1367,14 @@ Bool
 DRI2Authenticate(ClientPtr client, ScreenPtr pScreen, uint32_t magic)
 {
     DRI2ScreenPtr ds;
-    DRI2ClientPtr dri2_client = dri2ClientPrivate(client);
+    DRI2ClientPtr dri2_client;
     ScreenPtr primescreen;
 
+    if (!dixPrivateKeyRegistered(dri2ScreenPrivateKey))
+        return FALSE;
+
+    dri2_client = dri2ClientPrivate(client);
+
     ds = DRI2GetScreenPrime(pScreen, dri2_client->prime_id);
     if (ds == NULL)
         return FALSE;
@@ -1508,12 +1516,6 @@ DRI2ScreenInit(ScreenPtr pScreen, DRI2InfoPtr info)
     if (info->version < 3)
         return FALSE;
 
-    if (!xf86VGAarbiterAllowDRI(pScreen)) {
-        xf86DrvMsg(pScreen->myNum, X_WARNING,
-                   "[DRI2] Direct rendering is not supported when VGA arb is necessary for the device\n");
-        return FALSE;
-    }
-
     if (!dixRegisterPrivateKey(&dri2ScreenPrivateKeyRec, PRIVATE_SCREEN, 0))
         return FALSE;
 
@@ -1610,9 +1612,16 @@ DRI2ScreenInit(ScreenPtr pScreen, DRI2InfoPtr info)
         if (info->driverName) {
             ds->driverNames[0] = info->driverName;
         } else {
+            /* FIXME dri2_probe_driver_name() returns a strdup-ed string,
+             * currently this gets leaked */
             ds->driverNames[0] = ds->driverNames[1] = dri2_probe_driver_name(pScreen, info);
             if (!ds->driverNames[0])
                 return FALSE;
+
+            /* There is no VDPAU driver for i965, fallback to the generic
+             * OpenGL/VAAPI va_gl backend to emulate VDPAU on i965. */
+            if (strcmp(ds->driverNames[0], "i965") == 0)
+                ds->driverNames[1] = "va_gl";
         }
     }
     else {
